#ifndef CALC_H
#define CALC_H

#define NUMBER_TYPE   50
#define POINTER_TYPE  51

#define T_POWER 0
#define T_MULT  1
#define T_DIV   2
#define T_MOD   3
#define T_PLUS  4
#define T_MINUS 5
#define T_LE 6
#define T_LT 7
#define T_GE 8
#define T_GT 9
#define T_EQ 10
#define T_NEQ 11
#define T_AND 12
#define T_OR 13

#define TOP_BINARY 13

#define BR_CLOSE 14
#define BRACKET 15

#define T_SINH 16
#define T_COSH 17
#define T_TANH 18
#define T_SIN  19
#define T_COS  20
#define T_TAN  21
#define T_EXP  22
#define T_SQR  23
#define T_SQRT 24
#define T_THETA 25
#define T_SIGMA  26
#define T_LOG   27
#define T_LOG10 28
#define T_CUBE  29
#define T_ATAN  30
#define T_INT  31
#define T_NOT  32

#define T_UNARY_NOT  33
#define T_UNARY_PLUS  34
#define T_UNARY_MINUS 35

#define TYPE_NUM 36

const char priority[TOP_BINARY+1] = { 0, 1, 1, 1, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6 };
const char priorityLevels = priority[TOP_BINARY] + 1;

//*** make sure a token is not a prefix of any token further in the type_id array (otherwise make it "special" token)

const char type_id[TYPE_NUM][7] = { "^", "*", "/", "mod", "+", "-", "<=",
				    "<", ">=", ">", "==", "!=", "and", "or", ")", "(",
                        "sinh(", "cosh(", "tanh(", "sin(", "cos(", "tan(", 
                        "exp(", "sqr(", "sqrt(", "theta(", "sigma(", "log(", "log10(", "cube(", 
                        "atan(", "int(", "not(", "!", "+", "-" };

inline bool isBinary(char tp) { if ( tp <= TOP_BINARY ) return true; else return false; }
inline bool isUnary(char tp)  { if ( tp >= T_UNARY_NOT && tp < TYPE_NUM ) return true; else return false; }
inline bool isOperand(char tp)    { return ((tp == NUMBER_TYPE) || (tp == POINTER_TYPE) || (tp == BR_CLOSE )) ? true : false; }
inline bool isFunction(char tp)   { return ( (tp >= BRACKET) && (tp <= T_NOT) ) ?  true : false; }

inline bool isExpressionStart(char *s) {
  if ( equal(s,ASSIGN_LABEL) || equal(s,VAR_LABEL) || equal(s,ODE_LABEL) || equal(s,IF_TOKEN) || 
       equal(s, "print" ) || equal(s,"append") || equal(s,"current") || equal(s,"tortuosity") ) 
  return true; else return false; }

inline bool isExpressionStop(char *s) {
  if (equal(s,CARR_RET_TOKEN) || equal(s,"to") || equal(s,"step") || equal(s, "then" ) || equal(s, "," ) ) return true; else return false;}

int get_max_term_num(TokenString &, int);

double *ResolveID(char *, double ** = 0, FieldObj *Ca = 0, BufferArray *Bufs = 0, class KineticObj *KO = 0, 
       const char *message=0,
       double * = 0, const char * = "t", double * = 0, const char * = "%", double * = 0, const char * = "%");

const char   *ResolvePtr(double *ptr, FieldObj *Ca, BufferArray *Bufs, KineticObj *KO,
       double * = 0, const char * = "t", double * = 0, const char * = "%", double * = 0, const char * = "%");

bool isFloat(const char *s);
bool isInt(const char *s);

//*************************************************************************************

struct TermStruct
{
  char   type;
  double val;
  double *ptr;
};

//*************************************************************************************

class ExpressionObj
{

 public:

  struct TermStruct *term_array;
  int    term_num;
  char   formula[1024];

  ExpressionObj(int n = 0)  { term_array = new TermStruct[term_num = n]; strcpy(formula,""); }

  ExpressionObj(const ExpressionObj &EO) { 
    term_array = new TermStruct[term_num = EO.term_num];
    *this = EO;
  }

  void operator=(const ExpressionObj &EO) {
    for (int i = 0; i < term_num; i++) term_array[i] = EO.term_array[i]; }

  ~ExpressionObj()    { delete [] term_array; }

  int    eliminate(int, int);
  void   pushOp(char op, char *opStack, int *indStack, int &stackHead, int &j);
  void   Optimize(int * = 0);
  double Evaluate(int * = 0);
  void   print(FILE *f = stderr);

  friend class KinExpression;

  void  buildFormulaString(FieldObj *Ca, BufferArray *Bufs, KineticObj *KO,
        double * = 0, const char * = "t", double * = 0, const char * = "%", double * = 0, const char * = "%",
        const char * errorMessage = "");
};

//*************************************************************************************

class KinExpression : public ExpressionObj
{
 public:

  KinExpression(TokenString &, int,
                const char * message = ">>> Syntax error in expression",
                double * = 0, const char * = "t", double * = 0, const char * = "%", double * = 0, const char * = "%",  
                class FieldObj * = 0, class BufferArray * = 0, class KineticObj * = 0,  
                int *pLast = 0);

  KinExpression() : ExpressionObj(0) { } 
};

//*************************************************************************************

#endif
